import json

from PyQt6.QtCore import Qt
from PyQt6.QtWidgets import QSizePolicy
from PyQtUIkit.core import KitFont
from PyQtUIkit.widgets import *
from qasync import asyncSlot

from StdioBridge._ui.expandable_widget import ExpandableWidget
from StdioBridge.client import Client


class Endpoint:
    def __init__(self, url: str, method: str, data: dict):
        self.url = url
        self.method = method
        self.body = data.get('body')
        self.path = data.get('path')
        self.query = data.get('query')


class ParameterItem(KitHBoxLayout):
    def __init__(self, name: str, param_type: str):
        super().__init__()
        self.spacing = 10
        left_layout = KitVBoxLayout()
        left_layout.spacing = 6
        self.addWidget(left_layout)

        left_layout.addWidget(label := KitLabel(name))
        label.font = 'bold'
        left_layout.addWidget(KitLabel(param_type))

        self._line_edit = KitLineEdit()
        self._line_edit.setFixedHeight(30)
        self.addWidget(self._line_edit)

    @property
    def value(self):
        return self._line_edit.text


class EndpointItem(ExpandableWidget):
    def __init__(self, client: Client, endpoint: Endpoint):
        super().__init__()
        self.client = client
        self.endpoint = endpoint
        self.path_params = dict()
        self.query_params = dict()

        self.top_layout.spacing = 10

        layout = KitHBoxLayout()
        layout.setFixedWidth(80)
        layout.main_palette = {
            'get': 'Menu',
            'post': 'Success',
            'put': 'Warning',
            'delete': 'Danger',
            'patch': 'Warning',
        }[endpoint.method]
        self.top_layout.addWidget(layout)

        method_label = KitLabel(endpoint.method.upper())
        method_label.font_size = KitFont.Size.BIG
        method_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        layout.addWidget(method_label)

        url_label = KitLabel(endpoint.url)
        self.top_layout.addWidget(url_label)

        self.body_layout.padding = 10
        self.body_layout.spacing = 6
        self.body_layout.alignment = Qt.AlignmentFlag.AlignTop
        self.body_layout.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Fixed)

        if endpoint.path:
            self.body_layout.addWidget(label := KitLabel("Path parameters:"))
            label.font_size = KitFont.Size.BIG

            for key, item in self.endpoint.path.items():
                widget = ParameterItem(key, item)
                self.body_layout.addWidget(widget)
                self.path_params[key] = widget

        if endpoint.query:
            self.body_layout.addWidget(label := KitLabel("Query parameters:"))
            label.font_size = KitFont.Size.BIG

            for key, item in self.endpoint.query.items():
                widget = ParameterItem(key, item)
                self.body_layout.addWidget(widget)
                self.query_params[key] = widget

        if endpoint.body is not None:
            self.body_layout.addWidget(label := KitLabel("Request body:"))
            label.font_size = KitFont.Size.BIG

            self._body_edit = KitTextEdit()
            self._body_edit.setFixedHeight(200)
            self._body_edit.font = 'mono'
            self.body_layout.addWidget(self._body_edit)
        else:
            self._body_edit = None

        self.body_layout.addWidget(KitHSeparator())

        button = KitButton("Execute")
        button.font_size = KitFont.Size.BIG
        button.on_click = lambda: self.execute()
        self.body_layout.addWidget(button)

        self.body_layout.addWidget(KitHSeparator())

        layout = KitHBoxLayout()
        self.body_layout.addWidget(layout)
        layout.spacing = 6
        layout.addWidget(KitLabel("Code: "))
        self._code_label = KitLabel("?")
        self._code_label.font_size = KitFont.Size.BIG
        layout.addWidget(self._code_label, 100)

        self._response_body_widget = KitTextEdit()
        self._response_body_widget.font = 'mono'
        self._response_body_widget.setReadOnly(True)
        self._response_body_widget.setFixedHeight(200)
        self.body_layout.addWidget(self._response_body_widget)

    @asyncSlot()
    async def execute(self):
        url = self.endpoint.url
        for key, item in self.path_params.items():
            url = url.replace(f'{{{key}}}', item.value)
        i = 0
        for key, item in self.query_params.items():
            url += f"{'?' if i == 0 else '&'}{key}={item.value}"
            i += 1

        if isinstance(self._body_edit, KitTextEdit) and self._body_edit.toPlainText().strip():
            body = json.loads(self._body_edit.toPlainText())
        else:
            body = None
        resp = await self.client._request(self.endpoint.method, url, body, stream=True)
        self._code_label.text = str(resp.code)
        self._response_body_widget.setText('')
        text = ''
        async for el in resp:
            text += json.dumps(el, indent=2)
            text += '\n'
            self._response_body_widget.setText(text)
        self._code_label.text = str(resp.code)



from PyQtUIkit.widgets import *


class ExpandableWidget(KitVBoxLayout):
    def __init__(self):
        super().__init__()
        self.__expanded = False

        top_layout = KitLayoutButton()
        top_layout.spacing = 6
        top_layout.padding = 5
        top_layout.setFixedHeight(38)
        top_layout.on_click = self.__on_click
        self.addWidget(top_layout)

        self._top_layout = KitHBoxLayout()
        top_layout.addWidget(self._top_layout)

        self._icon_down = KitIconWidget('line-chevron-down')
        self._icon_down.setFixedSize(28, 28)
        top_layout.addWidget(self._icon_down)

        self._icon_up = KitIconWidget('line-chevron-up')
        self._icon_up.setFixedSize(28, 28)
        self._icon_up.hide()
        top_layout.addWidget(self._icon_up)

        self._body_layout = KitVBoxLayout()
        self._body_layout.hide()
        self.addWidget(self._body_layout)

    @property
    def top_layout(self):
        return self._top_layout

    @property
    def body_layout(self):
        return self._body_layout

    @property
    def expanded(self):
        return self.__expanded

    def __on_click(self):
        if self.__expanded:
            self.collapse()
        else:
            self.expand()

    def expand(self):
        self.__expanded = True
        self._icon_down.hide()
        self._icon_up.show()
        self._body_layout.show()

    def collapse(self):
        self.__expanded = False
        self._icon_up.hide()
        self._icon_down.show()
        self._body_layout.hide()





import sys
from sys import argv

from PyQt6.QtCore import Qt
from PyQtUIkit.widgets import *
from qasync import asyncSlot

from StdioBridge._ui.request_item import Endpoint, EndpointItem
from StdioBridge._ui.search import search
from StdioBridge.client import Client


class MainWindow(KitMainWindow):
    def __init__(self):
        super().__init__()
        self.resize(800, 600)

        if len(argv) > 1:
            args = argv[1:]
        elif file := search():
            args = ['python' if sys.platform == 'win32' else 'python3', file]
        else:
            raise Exception("Api not found")
        self._client = Client(args, shell=True)

        main_layout = KitVBoxLayout()
        self.setCentralWidget(main_layout)

        scroll_area = KitScrollArea()
        main_layout.addWidget(scroll_area)

        self._scroll_layout = KitVBoxLayout()
        self._scroll_layout.alignment = Qt.AlignmentFlag.AlignTop
        self._scroll_layout.padding = 10
        self._scroll_layout.spacing = 10
        scroll_area.setWidget(self._scroll_layout)

    def showEvent(self, a0):
        super().showEvent(a0)
        self._load()

    @asyncSlot()
    async def _load(self):
        docs = await self._client.get('docs')
        docs = docs.data
        for key, items in docs.items():
            for method, item in items.items():
                endpoint = Endpoint(key, method, item)
                self._scroll_layout.addWidget(EndpointItem(self._client, endpoint))


def main():
    KitAsyncApplication(MainWindow).exec()


if __name__ == '__main__':
    main()



import os


def search():
    for root, dirs, files in os.walk('.'):
        for file in files:
            if file.endswith('.py'):
                text = open(os.path.join(root, file), 'r').read()
                if ('from StdioBridge' in text or 'import StdioBridge' in text) and 'BridgeAPI' in text:
                    return os.path.join(root, file)
